#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Foobar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFCore.utils import getToolByName

from Products.Quills.config import *
from Products.Quills.properties import props
from Products.Quills import QuillsWorkflow
from Products.Quills.permissions import \
    setupPortalSecurity, unsetupPortalSecurity
from Products.Quills.migrations import quills09to15
from Products.Quills import MetaWeblogAPI

from StringIO import StringIO


def install(self):
    """Install Quills."""
    out = StringIO()

    # Try to install dependencies
    try:
        from Products.Quills.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        get_transaction().commit(1)

    portal = getToolByName(self, 'portal_url').getPortalObject()
    # Add the quills_tool
    if not 'quills_tool' in portal.objectIds():
        self.manage_addProduct['Quills'].manage_addTool('Quills Tool', None)
        print >> out, "Added quills tool to the portal."

    # Register CSS
    registerStylesheets(self, out)

    # Add the site administration interface
    # XXX disabling till we have some config options
    #portal_controlpanel = getToolByName(self, 'portal_controlpanel')
    #portal_controlpanel.registerConfiglets(configlets)

    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    prop_tool = getToolByName(self, 'portal_properties')
    if 'quills_properties' not in prop_tool.objectIds():
        prop_tool.manage_addPropertySheet('quills_properties',
                'Quills Properties')
        print >> out, "Added quills property sheet."
    quills_props = prop_tool.quills_properties

    for item in props:
        if quills_props.hasProperty(item[0]):
            quills_props._updateProperty(item[0], item[1])
        else:
            quills_props._setProperty(item[0], item[1], item[2])

    # Create an RPCAuth if there is not one already
    installRPCAuth(self, out)
    # Keep some of our types out of the navtree
    updateNavtreeProperties(self, out)
    # Add the special weblog author role and setup its permissions.
    setupPortalSecurity(portal, out)
    # Allow Weblogs to be used as the front page of a site
    # FIXME: When this is enabled there is no way to add an Entry to the blog
    #updateDefaultPageTypes(self)

    # Setup workflow
    installWorkflow(self, out)
    updateSchemas(self, out)
    automigrate(self, out)
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()

def installWorkflow(self, out):
    QuillsWorkflow.install()
    wf_tool = getToolByName(self, 'portal_workflow')
    if not 'quills_workflow' in wf_tool.objectIds():
         wf_tool.manage_addWorkflow('quills_workflow (Quills Workflow)',
                                    'quills_workflow')
         print >> out, "Added quills workflow."
    wf = wf_tool.getWorkflowById('quills_workflow')
    for p in ['moveToArchive', 'moveToWeblogRoot']:
        if not p in wf.scripts.objectIds():
            factory = wf.scripts.manage_addProduct['ExternalMethod']
            factory.manage_addExternalMethod(p, p, 
                                             'Quills.workflow_scripts', p)

    wf_tool.updateRoleMappings()
    wf_tool.setChainForPortalTypes(('WeblogEntry',),
                                   chain='quills_workflow')
    print >> out, 'Set quills_workflow as default for WeblogEntry.'

def installRPCAuth(self, out):
    """Install RPCAuth object at site root.

    It catches the exception when RPCAuth is not available and only
    prints a warning, let others get to the admin.  
    """

    if not hasattr(self, 'RPCAuth'):
        try:
            self.manage_addProduct['RPCAuth'].manage_addRPCAuth('RPCAuth')
            print >> out, "Added RPCAuth to the portal root."
        except:
            print >> out, "An RPCAuth instance could not be created."
            print >> out, \
                "Please make sure RPCAuth is installed correctly."
    # Get hold of the RPCAuth object on the portal
    rpcauth = getattr(self, 'RPCAuth')
    # We now add the authentication provider information to RPCAuth here,
    # in the install script, rather than in Weblog.initializeArchetype.
    # There is no need for every Weblog to set this up as it only needs to
    # done once at the portal root.  If we do *not* do it here during
    # installation, then doing 'uninstall' followed by 'install' in the QI
    # tool will leave you with an unconfigured RPCAuth object because the
    # first step removes all objects added to the portal at install time,
    # while the second adds back an unconfigured RPCAuth object.
    rpcauth.addAuthProvider(MetaWeblogAPI.authMethods,
                            MetaWeblogAPI.genericMetaWeblogAuth)

def updateNavtreeProperties(self, out):
    """Keep some content types from showing up in the navigation tree.
    """

    ptool = getToolByName(self, 'portal_properties')
    dontList = list(
        ptool.navtree_properties.getProperty('metaTypesNotToList'))
    newDontList = dontList[:] # hard copy
     # DONT_LIST_IN_NAVTREE is defined in ../config.py.
    for item in DONT_LIST_IN_NAVTREE:
        if item not in dontList:
            newDontList.append(item)
    if newDontList != dontList:
        ptool.navtree_properties._updateProperty('metaTypesNotToList',
                                                 newDontList)
        print >> out, "Added to metaTypesNotToList."

def updateDefaultPageTypes(self, out):
    """Allow a Weblog to be used on the front page of the site.
    """

    # Temporarily disabled!

    ptool = getToolByName(self, 'portal_properties')
    defaultPageTypes = ptool.site_properties.getProperty('default_page_types')
    new_unlisted = []
    # make it mutable
    types = [item for item in defaultPageTypes]
    if 'Weblog' not in types:
        types.append('Weblog')
    ptool.site_properties._updateProperty('default_page_types', types)

def _updateResources(tool, updates):
    # Lifted from DIYplonestyle
    resources = list(tool.resources)
    for update in updates:
        resource = tool.getResource(update['id'])
        for key in [k for k in update.keys() if k != 'id']:
            # keep a trace of our customization for further reversion
            if not resource._data.has_key('original_'+key):
                resource._data['original_'+key] = resource._data[key]
            resource._data[key] = update[key]
        tool.cookResources()

def registerStylesheets(self, out):
    """Register CSS with the registry.
    """

    csstool = getToolByName(self, 'portal_css')
    existing = csstool.getResourceIds()
    updates = []
    for css in STYLESHEETS:
        if not css.get('id') in existing:
            csstool.registerStylesheet(**css)
            print >> out, "Registered stylesheet %s." % css.get('id')
        else:
            updates.append(css)
    if updates:
        _updateResources(csstool, updates)

def automigrate(self, out):
    """Call the migration in so far it can be done safely.

    The 0.9 to 1.5 migration only removes old stuff we don't need
    anymore (which is always safe, as we check for the presence of it
    before removing it) and it migrates some old attributes when
    found. That's all safe (and unittested). :-)
    """

    migration = quills09to15.Migration(site=self,
                                       out=out)
    migration.migrate()
    
def updateSchemas(self, out):
    """Update the schemas.
    """

    at = getToolByName(self, 'archetype_tool')
    class dummy:
        form = {}
    dummyRequest = dummy()
    contentTypes = ['Weblog',
                    'WeblogEntry',
                    'WeblogArchive']
    for contentType in contentTypes:
        name = 'Quills.%s' % contentType
        print >> out, "Migrating schema for %s." % name
        dummyRequest.form[name] = 1
    at.manage_updateSchema(update_all=1,
                           REQUEST=dummyRequest)

def unregisterStylesheets(self):
    """Remove Quills CSS from the registry.
    """

    registry = getToolByName(self, 'portal_css')
    registry.unregisterResource('Quills.css')

def uninstall(self):
    """Uninstall Quills."""
    out = StringIO()

    portal_controlpanel = getToolByName(self, 'portal_controlpanel')
    portal_controlpanel.unregisterApplication('Quills')
    unsetupPortalSecurity(self, out)
    unregisterStylesheets(self)

    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()

