#
# Authors: Tom von Schwerdtner <tvon@etria.com>
#          Brian Skahan <bskahan@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# CMF imports
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName

# Zope 2 imports
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass

# Standard library imports
import re
import urllib

# Plone imports
from Products.CMFPlone.PloneBatch import Batch as PloneBatch

# Product imports
#from topic import Topic


class QuillsTool (UniqueObject, SimpleItem):
    """A set of utility methods for Quills"""

    id = 'quills_tool'
    meta_type= 'Quills Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    security.declarePublic('getByUID')
    def getByUID(self, uid):
        """Shortcut method for the [Blogger,MetaWeblog]API code"""
        uid_catalog = getToolByName(self, 'uid_catalog')
        lazy_cat = uid_catalog(UID=uid)
        o = lazy_cat[0].getObject()
        return o

    security.declarePublic('getWeblogEntriesDates')
    def getWeblogEntriesDates(self, entries_dict):
        """Return a sorted list of dates for dict of entries"""
        days = entries_dict.keys()
        days.sort()
        days.reverse()
        return days

    security.declarePublic('sortWeblogEntriesToDates')
    def sortWeblogEntriesToDates(self, lazy_entries, resolution='day'):
        """Sort the lazy entries into a dictionary keyed on dates
        with values a lists of WeblogEntry (catalog brains) that
        were posted on those days.
        The default 'resolution' is 'day', meaning the returned dictionary
        will be keyed on individual days.  Other options are 'month' and 'year'.
        """
        if resolution == 'day':
            format = '%Y-%m-%d 00:00:00'
        elif resolution == 'month':
            format = '%Y-%m'
        elif resolution == 'year':
            format = '%Y'
        else:
            msg = "The 'resolution' parameter must be one of 'day', 'month', or 'year'.  You passed %s."
            raise Exception(msg % resolution)
        if isinstance(lazy_entries, PloneBatch):
            start = lazy_entries.start - 1
            end = lazy_entries.end
            lazy_entries = lazy_entries._sequence[start:end]
        results = {}
        for lazy_entry in lazy_entries:
            date = lazy_entry.effective.strftime(format)
            try:
                if results[date]:
                    # Add the entry to the top of the list for that day
                    results[date].append(lazy_entry)
            except:
                results[date] = [lazy_entry,]
        return results


    security.declarePublic('getParentWeblog')
    def getParentWeblog(self, item):
        """Return a Weblog instance above the current object."""
        parent = item.getParentNode()

        if item.meta_type == 'Weblog':
            return item
        elif parent.meta_type == 'Weblog':
            return parent
        elif parent.meta_type == 'Plone Site':
            return None
        else:
            return self.getParentWeblog(parent)

        return None

    security.declarePublic('findRPCAuth')
    def findRPCAuth(self, parent):
        """Return an RPCAuth instance through traversal if available."""
        while hasattr(parent,'aq_parent'):
            RPCAuths = parent.objectValues('RPC Auth')
            for RPCAuth in RPCAuths:
                return RPCAuth
            parent = parent.aq_parent
        return None

InitializeClass(QuillsTool)
