# Zope3 imports
from zope.interface import Interface


class IWeblog(Interface):
    """A weblog.
    """

    def getTopics():
        """Return a sequence of ITopic instances for this blog.
        """

    def getAuthors():
        """Return a sequence of every portal member id that has posted to this
        weblog.
        """

    def getLazyEntries():
        """Return all entries that are in the published state.
        """


class IWeblogEntry(Interface):
    """A weblog entry.
    """

    def getCategories():
        """Return the categories/topics for this entry.
        """

    def getExcerpt():
        """Return an excerpt/description of the post.
        """

    def getText():
        """Return the HTML text body of the post.
        """


class ITopic(Interface):
    """A keyword, with an optionally associated image, that knows
    how to find content tagged with it.
    """

    def getId():
        """Return the keyword/id for this topic.
        """

    def getTitle():
        """Return title.
        """

    def title_or_id():
        """Return the title, if not empty, else the id.
        """

    def getDescription():
        """Return the description for this topic.
        """

    def getImage():
        """Return an image object for this topic/keyword.
        """

    def getLazyEntries():
        """Return all those entries that are tagged with this topic/keyword
        and that are in the published state.
        """

    def getLastModified():
        """Return a DateTime instance for when a post in this ITopic was
        last modified.
        """

    def __len__():
        """Return the number of posts for this ITopic.
        """

    def __str__():
        """Return a string representation of one or more the keywords.
        """


class IAuthorsTopic(ITopic):
    """Represents a topic that queries based on authors (aka 'Creators').
    """
