# zope 3 imports
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import getToolByName

# Five imports
from Products.Five.browser import BrowserView

# Product imports
from Products.plonetrackback.interfaces import ITrackback, \
     IReadTrackbackInManager, ITrackbackInManager, \
     ITrackbackInManagerView, ITrackbackInEditView
from Products.plonetrackback.config import TOOL_ID
from Products.plonetrackback.errors import TrackbackError


class TrackbackInManagerView(BrowserView):

    implements(ITrackbackInManagerView, IReadTrackbackInManager)

    def __init__(self, context, request):
        # Could just adapt to IReadTrackbackInManager, but we may want
        # more than that at some point, so we just use the full interface.
        self.tbim = ITrackbackInManager(context)
        return super(TrackbackInManagerView, self).__init__(context, request)

    def receiveTrackback(self):
        """See ITrackbackInManagerView.
        """
        try:
            trackback = ITrackback(self.request)
            self.tbim.receive(trackback)
        except TrackbackError, e:
            code = e.code
            message = e.message
            return self.formatErrorResponse(code, message)

    def formatErrorResponse(self, code, message):
        """See ITrackbackInManagerView.
        """
        self.request.RESPONSE.setHeader('Content-Type','text/xml')
        resp = []
        resp.append( '<?xml version="1.0" encoding="iso-8859-1"?>' )
        resp.append( '<response>' )
        resp.append( '<error>%s</error>' % str(code) )
        if message != '':
            resp.append( '<message>%s</message>' % message )
        resp.append( '</response>' )
        return '\n'.join(resp)

    def trackbackURI(self):
        """See ITrackbackInManagerView.
        """
        return '%s/trackback' % self.context.absolute_url()

    def getTrackbacks(self):
        """See IReadTrackbackInManagerView.
        """
        return self.tbim.getTrackbacks()

    def __getitem__(self, key):
        """See IReadTrackbackInManagerView.
        """
        return self.tbim[key]


class TrackbackInEditView(BrowserView):

    implements(ITrackbackInEditView)

    def __init__(self, context, request):
        # Could just adapt to IReadTrackbackInManager, but we may want
        # more than that at some point, so we just use the full interface.
        self.tbim = ITrackbackInManager(context)
        return super(TrackbackInEditView, self).__init__(context, request)

    def deleteTrackbacks(self, ids=None):
        """See ITrackbackInEditView)
        """
        if not ids:
            return
        for id in ids:
            self.tbim.deleteTrackback(id)
        # XXX Could/should add some kind of status message here to indicate
        # that trackbacks have been deleted.
        self.request.RESPONSE.redirect(self.request['HTTP_REFERER'])
