# Zope 3 imports
from zope.interface import Interface


class ITrackback(Interface):
    """Represents a trackback.
    """

    def getTitle():
        """Return the title.
        """

    def getExcerpt():
        """Return the excerpt that will be sent as part of the ping.
        """

    def getURL():
        """Return the URL of the trackback source.
        """

    def getSiteTitle():
        """Return the site/blog title.
        """


class IStoredTrackback(ITrackback):
    """Represents a stored/persistent trackback.
    """

    def getDatetimeReceived():
        """Return a datetime object representing when the trackback was
        received.
        """

    def getId():
        """Return the Id, acts like an id in a folder.
        """

    def getPath():
        """Return the object path to this stored trackback.
        """


class IReadTrackbackInManager(Interface):
    """Management of incoming trackback pings for an object.
    """

    def getTrackbacks():
        """Return a sequence of ITrackback objects.
        """

    #def __getitem__(key):
    #    """Support dictionary lookup of stored trackbacks.
    #    """


class IWriteTrackbackInManager(Interface):
    """Management of incoming trackback pings for an object.
    """

    def storeTrackback(trackback):
        """Store the ITrackback object.
        """

    def deleteTrackback(trackbackNumber):
        """Delete the trackback with the trackbackNumber (aka id).
        """

    def receive(trackback):
        """Handle the incoming ITrackback ping.
        """


class ITrackbackInManager(IReadTrackbackInManager, IWriteTrackbackInManager):
    """Management of incoming trackback pings for an object.
    """


class ITrackbackOutManager(ITrackback):
    """Used for objects that will be the source of a trackback
    ping.  i.e. objects that want to inform remote objects of
    their existence.
    """

    def getURLsToPing():
        """Return the sequence of URLs to send trackback pings to.
        """

    def setURLsToPing(urls):
        """Set the sequence of URLs to send trackback pings to.
        """

    def getPingedURLs():
        """Return the sequence of URLs that have already been pinged.
        """

    def setPingedURLs(urls):
        """Set the sequence of URLs that have been pinged.
        """

    def sendTrackbacks():
        """Send trackback pings for context.
        """


class ITrackbackOutManagerView(Interface):
    """XXX Document me!
    """

    def editTrackbacks(unpinged_urls=None):
        """
        """

    def getEditURL():
        """Returns the URL that exposes the self.editTrackbacks method for context.
        """

    def getTrackbackOutManager():
        """Returns the ITrackbackOutManager for context.
        Hung here to make it easy to get hold of with TTW code.
        """

    def sendTrackbacks(unpinged_urls=None):
        """XXX Fill me in.
        """

    def getURLsToPingAsLines():
        """
        """


class ITrackbackInManagerView(Interface):
    """
    """

    def receiveTrackback():
        """Handle incoming trackback pings from the request.
        """

    def formatErrorResponse(code, message):
        """Format the error response.
        """

    def trackbackURI():
        """Return the URI for sending trackback pings to context.
        """

    def __getitem__(key):
        """Dictionary lookup supported for contained trackbacks.
        """


class ITrackbackInEditView(Interface):
    """
    """

    def deleteTrackbacks(ids=None):
        """Delete the sequence of trackback ids that are passed in
        (probably by calling ITrackbackInManager.deleteTrackback for each
        id).
        """
