# zope 3 imports
from zope.interface import implements

# zope 2 imports
from Acquisition import Implicit
from persistent import Persistent
from DateTime.DateTime import DateTime

# Product imports
from interfaces import ITrackback, IStoredTrackback
from errors import TrackbackError


class Trackback(Implicit):

    implements(ITrackback)

    def __init__(self, title=None, url=None, excerpt=None, site_title=None):
        self._title = title
        self._url = url
        self._excerpt = excerpt
        self._site_title = site_title

    def getTitle(self):
        """See ITrackback.
        """
        return self._title

    def getURL(self):
        """See ITrackback.
        """
        return self._url

    def getExcerpt(self):
        """See ITrackback.
        """
        return self._excerpt

    def getSiteTitle(self):
        """See ITrackback.
        """
        return self._site_title


class StoredTrackback(Trackback, Persistent):
    """Persistent trackback object.
    """

    implements(IStoredTrackback)

    def __init__(self, id, trackback=None, **kw):
        self.id = str(id)
        self._datetime_received = kw.get( 'datetime', DateTime() )
        if trackback is not None:
            kw['title'] = trackback.getTitle()
            kw['url'] = trackback.getURL()
            kw['excerpt'] = trackback.getExcerpt()
            kw['site_title'] = trackback.getSiteTitle()
        return super(StoredTrackback, self).__init__(**kw)

    def getId(self):
        """See IStoredTrackback.
        """
        return str(self.id)

    def getDatetimeReceived(self):
        """See IStoredTrackback.
        """
        return self._datetime_received

    def getPath(self):
        """See IStoredTrackback.
        """
        # XXX Hardcoding @@TrackbackInManagerView smells wrong. Should
        # probably lookup the view name for the interface... if I knew how.
        format = '%s/@@TrackbackInManagerView/%s'
        parent_path = self.aq_parent.absolute_url()
        return format % ( parent_path, self.getId() )


def trackbackFromRequest(request):
    """Factory for getting an ITrackback from the request object.
    """
    if not request.has_key('url'):
        code = 0
        message = "Bad request. Must send a 'url' in the trackback ping."
        raise TrackbackError(code, message)
    tbdict = {  'title' : request.get('title', '')
              , 'excerpt' : request.get('excerpt', '')
              , 'url'     : request.get('url', '')
              , 'site_title' : request.get('blog_name', '')
             }
    return Trackback(**tbdict)

