# zope 3 imports
from zope.interface import implements
from zope.app.annotation.interfaces import IAnnotations, IAnnotatable
from persistent.mapping import  PersistentMapping

# zope 2 imports
from Acquisition import Implicit
#from Globals import InitializeClass
#from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from interfaces import ITrackbackInManager, ITrackback
from errors import TrackbackError, BlacklistedURL
from data import checkTrackback
from trackback import Trackback
from config import TOOL_ID
from trackback import StoredTrackback


TRACKBACK_KEY = 'plonetrackback.receiver'

def decorateAsPloneContent(obj):
    # XXX Yuck!  All this nonsense gets persisted until the catalog learns
    # to adapt objects that it is indexing and this all moves to an adapter.
    obj.Title = obj.getTitle()
    obj.portal_type = ''
    # Is it ok to 'lie' here?
    obj.meta_type = 'Trackback'
    obj.Type = ''
    # What is this madness?  Why does Plone have two of all the dates?
    obj.modified = obj.ModificationDate = obj.getDatetimeReceived()
    obj.created = obj.CreationDate = obj.getDatetimeReceived()
    obj.effective = obj.EffectiveDate = None
    obj.getIcon = ''
    obj.start = None
    obj.end = None
    obj.Creator = None
    obj.listCreators = []
    # Play nicely with the Link content type
    obj.getRemoteUrl = obj.getURL()


class BaseTrackbackInManager:

    def receive(self, trackback):
        """See ITrackbackInManager.
        """
        try:
            checkTrackback(trackback)
        except BlacklistedURL, e:
            raise TrackbackError(code=0, message='Blacklisted')
        self.storeTrackback(trackback)


class TrackbackInManagerAnnotations(BaseTrackbackInManager):
    """Adapter to store/retrieve trackbacks with IAnnotations.
    """

    implements(ITrackbackInManager)
    __used_for__ = IAnnotatable

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(context)
        trackbacks = annotations.get(TRACKBACK_KEY, None)
        if trackbacks is None:
            annotations[TRACKBACK_KEY] = PersistentMapping()
        self._trackbacks = annotations[TRACKBACK_KEY]
    
    def storeTrackback(self, trackback):
        """See IWriteTrackbackInManager.
        """
        id = self._genTrackbackId()
        ptrackback = StoredTrackback(id, trackback=trackback)
        # XXX Ack!  See note above.  This should go!
        decorateAsPloneContent(ptrackback)
        self._trackbacks[id] = ptrackback
        # Wrap ptrackback after storing it so that the cataloging can work.
        wptrackback = ptrackback.__of__(self.context)
        catalog = getToolByName(self.context, 'portal_catalog')
        # XXX What happens if the object the this trackback is annotated to
        # moves?  We would then need to reindex for the new path.  Perhaps
        # we should use an event handler for the new ObjectMoved events
        # that Zope sends out?
        catalog.catalog_object( wptrackback, wptrackback.getPath() )

    def deleteTrackback(self, id):
        """See IWriteTrackbackInManager.
        """
        trackback = self[id]
        del self._trackbacks[id]
        catalog = getToolByName(self.context, 'portal_catalog')
        catalog.unindexObject(trackback)

    def getTrackbacks(self):
        """See IReadTrackbackInManager.
        """
        tbs =[]
        for each in self._trackbacks.values():
            tbs.append( each.__of__(self.context) )
        return tbs

    def __getitem__(self, key):
        """See IReadTrackbackInManager.
        """
        return self._trackbacks[key].__of__(self.context)

    def _genTrackbackId(self):
        """
        """
        trackbacks = self.getTrackbacks()
        id = len(trackbacks)
        while self._trackbacks.has_key(str(id)):
            id += 1
        return str(id)


class ImplicitTrackbackInManagerAnnotations(TrackbackInManagerAnnotations,
                                            Implicit):
    """AAAARRRGGHHH! I hate acquisition!
    """
    pass
